//
//  CompararCompleto.swift
//  Music-Mate-New-Design
//
//  Created by André Ricardo Marques Rogério on 01/10/18.
//  Copyright © 2018 LT Music Developer. All rights reserved.
//

import Foundation

/**
 * Faz uma comparação de um lista de notas preexistente e response se
 * cada uma das notas é foi tocada(True) ou não(False).
 */
class CompararCompleto {
    private weak var adicionarDados: EngineWaiter!
    private let gerenteDeAudio: GerenteDeAudio
    
    init(adicionarDados: EngineWaiter, gerenteDeAudio: GerenteDeAudio){
        self.adicionarDados = adicionarDados
        self.gerenteDeAudio = gerenteDeAudio
    }
    
    private let tag = "CompararCompleto"
    
    private let taxaDeAmostragem = Configuracoes.ComparadorCompleto.taxaAmostragem
    private let comprimentoFourier = Configuracoes.ComparadorCompleto.comprimentoFourier
    private let janelamentoSeg: Float = Float(Configuracoes.ComparadorCompleto.janelamentoMS) / 1000.0
    
    static var quantizacao = 2210
    
    private lazy var analisador = AnalisadorFrequencia(taxaDeAmostragem: taxaDeAmostragem, comprimentoFourier: comprimentoFourier)
    
    private var processando = false
    
    var estaProcessando: Bool { get { return processando} }
    
    
    private func analisar(notas: [EngineNota]) {
        processando = true
        
        print("Comecou perform 2!")
        adicionarDados.serve(status: Engine.Status.Comecou, "Comecando!")
        
        var notaIndex = 0
        while (processando) {
            let nota = notas[notaIndex]
            let buffer = gerenteDeAudio.valoresDe(inicio: nota.inicio, fim: nota.fim)
            
            if (buffer.count > 0) {
                print("Nota disponivel comeco: \(nota.inicio) fim: \(nota.fim)")
                let janelas = Vetor().dividir(entrada: buffer, compPedaco: Int(round(janelamentoSeg * Float(taxaDeAmostragem))), passo: nil)
                
                var frequencias = [[Pico]]()
                janelas.forEach { frequencias.append(analisador.frequenciasEm(entrada: $0)) }
                
                let frequenciasPorTempo = dividirTempo(frequencias: frequencias)
                
                let possibilidades = possiveisFrequencias(frequencias: frequenciasPorTempo)
                
                print(possibilidades)
                
//                let frequencias = analisador.frequenciasEm(entrada: buffer)
//                let maisAlto = frequencias.first ?? Pico(index: -1)
//
//                switch (maisAlto.filtro - nota.filtro) {
//                case _ where Configuracoes.ComparadorSimples.hamonicos.contains(maisAlto.filtro - nota.filtro):
//                    print("Nota correta!")
//                    adicionarDados.serve((notaIndex, true, nota.filtro + 24), Engine.Status.Dados)
//                default:
//                    print("Nota incorreta!")
//                    if maisAlto.index != -1 {
//                        print(maisAlto.filtro, maisAlto.energia)
//                        let real = teste(frequencias: frequencias, falso: maisAlto.filtro)
//
//                        if real {
//                            adicionarDados.serve((notaIndex, false, maisAlto.filtro + 12), Engine.Status.Dados)
//                        } else {
//                            adicionarDados.serve((notaIndex, false, maisAlto.filtro + 24), Engine.Status.Dados)
//                        }
//
//                    } else {
//                        adicionarDados.serve((notaIndex, false, -1), Engine.Status.Dados)
//                        print("Entrou errado")
//                    }
//
//                }
                
                notaIndex += 1
            } else {
                print("Nota indisponível")
                usleep(200000)
            }
            
            if (notaIndex >= notas.count) {
                print("Todas as notas foram analisadas!")
                processando = false
            }
        }
        
        adicionarDados.serve(status: Engine.Status.Terminou, "Terminado!")
        print("Terminou analise!")
    }
    
    private func dividirTempo(frequencias: [[Pico]]) -> [[Pico]] {
        //Tamanho de uma quantizacao em segundos.
        let quantizacao = Float(CompararCompleto.quantizacao) / Float(taxaDeAmostragem)

        //Tamanho de um tempo em quantidade de janelas.
        let qJanelasPorQuantizacao = quantizacao / janelamentoSeg
        
        let vezes = Int((Float(frequencias.count) / qJanelasPorQuantizacao))

        var saida = [[Pico]]()

        //Analisa cada quantizacao.
        for vez in 0..<vezes {
            //Primeira janela.
            let comeco = Int(Float(vez) * qJanelasPorQuantizacao)
            //Ultima janela.
            let fim = Int(Float(vez + 1) * qJanelasPorQuantizacao)
            
            var freqEmTempo = [Pico]()
            
            for index in comeco..<fim {
                freqEmTempo.append(contentsOf: frequencias[index])
            }
            
            //Remove frequencias repetidas.
            analisador.somaAmplitude(picos: &freqEmTempo)
            
            //Remove frequencias com energia baixa.
            analisador.dbDiference(picos: &freqEmTempo, limiteDb: -8)
            
            //Adiciona o tempo a lista.
            saida.append(freqEmTempo)
        }
    
        return saida
    }
    
    private func possiveisFrequencias(frequencias: [[Pico]]) -> [Int] {
        var possiveisNotas = [Int]()

        frequencias.forEach {
            $0.forEach {
                if !possiveisNotas.contains($0.filtro) {
                    possiveisNotas.append($0.filtro)
                }
            }
        }

        possiveisNotas.sort()
        
        return possiveisNotas
    }
    
    func comecar(notas: [EngineNota]) {
        if (estaProcessando) {
            print("Já está em processamento!")
            return
        }
        
        DispatchQueue.global(qos: .background).async {
            self.analisar(notas: notas)
        }
    }
    
    func interromper() {
        processando = false
    }
    
    func teste(frequencias: [Pico], falso: Int) -> Bool {
        for pico in frequencias {
            if (falso - 12 == pico.filtro) {
                return true
            }
        }
        
        return false
    }
}
