//
//  Compor.swift
//  Music-Mate-New-Design
//
//  Created by Raimundo Corrêa de Oliveira on 21/10/2018.
//  Copyright © 2018 LT Music Developer. All rights reserved.
//

import Foundation

class Compor {
    private let tag = "Compor"
    
    private weak var adicionarDados: EngineWaiter!
    private let gerenteDeAudio: GerenteDeAudio
    
    init (adicionarDados: EngineWaiter, gerenteDeAudio: GerenteDeAudio) {
        self.adicionarDados = adicionarDados
        self.gerenteDeAudio = gerenteDeAudio
    }
    
    private let taxaDeAmostragem = Configuracoes.ComparadorSimples.taxaAmostragem
    private let comprimentoFourier = Configuracoes.ComparadorSimples.comprimentoFourier
    private let janelamentoSeg = Float(Configuracoes.Compor.janelamentoMs) / Float(1000)
    
    private lazy var janelamento = Int(round(janelamentoSeg * Float(taxaDeAmostragem)))
    
    private class Compasso {
        var comeco = 0
        var fim = 0
        
        var tamanho: Int { get { return fim - comeco } }
        func mover(tamanho: Int) {
            comeco = fim
            fim += tamanho
        }
    }
    
    private lazy var conversor = ConversorTicks(notaBase: 960, metronomo: 120, taxaAmostragem: taxaDeAmostragem)
    
    private lazy var analisador = AnalisadorFrequencia(taxaDeAmostragem: taxaDeAmostragem, comprimentoFourier: comprimentoFourier)
    
    private var analisadorAtaque = AnalisadorAtaque()
    
    private var valorFinal = Int.max
    
    
    private(set) var estaProcessando = false
    
    private func janelasParaFloatArray(filtro: Int, janelas: [[Pico]]) -> [Float] {
        var saida = [Float](repeating: 0, count: janelas.count)
        
        for index in 0..<janelas.count {
            for pico in janelas[index] {
                if (pico.filtro == filtro) {
                    saida[index] = pico.energia
                }
            }
        }
        
        return saida
    }
    
    private func dividirTempo(tempo: TempoCompasso, frequencias: [[Pico]], duracaoCompasso: Int) -> [[Pico]] {
        let quantizacoes = tempo.numerador * 4
        //Tamanho de uma quantizacao em segundos.
        let quantizacao = Float(duracaoCompasso) / Float(quantizacoes * taxaDeAmostragem)
        
        //Tamanho de um tempo em quantidade de janelas.
        let qJanelasPorQuantizacao = quantizacao / janelamentoSeg
        
        var saida = [[Pico]]()
        
        //Analisa cada quantizacao.
        for i in 0..<quantizacoes {
            //Primeira janela.
            let comeco = Int(round(Float(i) * qJanelasPorQuantizacao))
            //Ultima janela.
            let fim = Int(round(Float(i + 1) * qJanelasPorQuantizacao))
            
            var freqEmTempo = [Pico]()
            
            for index in comeco..<fim where fim < frequencias.count {
                freqEmTempo.append(contentsOf: frequencias[index])
            }
            
            //Remove frequencias repetidas.
            analisador.somaAmplitude(picos: &freqEmTempo)
            
            //Remove frequencias com energia baixa.
            analisador.dbDiference(picos: &freqEmTempo, limiteDb: -8)
            
            //Adiciona o tempo a lista.
            saida.append(freqEmTempo)
        }
        
        return saida
    }
    
    private func possiveisFrequencias(frequencias: [[Pico]]) -> [Int] {
        var possiveisNotas = [Int]()
        
        //Pra cada quantização
        frequencias.forEach {
            //Verifica cada pico
            $0.forEach { pico in
                if (!possiveisNotas.contains(pico.filtro)) {
                    possiveisNotas.append(pico.filtro)
                }
            }
        }
        
        return possiveisNotas
    }
    
    private func monofonizar(notas: inout [(Int, Int, Int)]) {
        notas.sort { $0.1 < $1.1 }
        
        var monofonico = [(Int, Int, Int)]()
        var mesmoTempo = [(Int, Int, Int)]()
        
        var refId = 0
        while refId < notas.count {
            mesmoTempo.append(notas[refId])
            for andante in (refId + 1)..<notas.count {
                if ((notas[refId].1 >= notas[andante].1 && notas[refId].1 < notas[andante].2) ||
                    (notas[refId].2 > notas[andante].1 && notas[refId].2 <= notas[andante].2) ||
                    (notas[refId].1 < notas[andante].1 && notas[refId].2 > notas[andante].2)) {
                    mesmoTempo.append(notas[andante])
                }
            }
            
            mesmoTempo.sort {  $0.0 < $1.1  }
            
            refId += mesmoTempo.count
            
            monofonico.append(mesmoTempo.first!)
            
            mesmoTempo.removeAll()
        }
        
        notas.removeAll()
        notas.append(contentsOf: monofonico)
    }
    
    /**
     * Começa processo de composição com base em configurações de uma partitura.
     */
    private func compor(configuracoes: [ConfiguracaoCompasso]) {
        print(tag, "Começou composição!")
        adicionarDados.serve(status: Engine.Status.Comecou, "Comecando!")
        
        //Identifica um valor "infinito" para que o compose
        //so pare de processar com atuação externa.
        valorFinal = Int.max
        
        //O id do compasso sendo analisado.
        var compassoId = 0
        
        var compassoEmTicks = Compasso()
        var compassoEmAmostra = Compasso()
        
        for configId in 0..<configuracoes.count {
            let configAtual = configuracoes[configId]
            //Se não houver próxima configuração, utiliza a quantidade de compassos como base.
            let configFutura = (configId + 1 < configuracoes.count) ? configuracoes[configId + 1] :
                ConfiguracaoCompasso(
                    compasso: 1000000,
                    notaBase: 0,
                    metronomo: 0,
                    tempoCompasso: TempoCompasso(numerador: 4,
                                                 denominador: 4))
            
            //Número de compassos com a mesma configuração atual.
            let qCompassos = configFutura.compasso - configAtual.compasso
            //Atualiza o conversor para as configurações.
            conversor.alterar(notaBase: configAtual.notaBase, metronomo: configAtual.metronomo)
            
            finalizar: for _ in 0..<qCompassos {
                let tamanhoTicks = Int(round(configAtual.tempoCompasso.razao * Float(3840)))
                compassoEmTicks.mover(tamanho: tamanhoTicks)
                compassoEmAmostra.mover(tamanho: conversor.ticksParaAmostras(ticks: tamanhoTicks))
                var buffer = [Float]()
                repeat {
                    buffer = gerenteDeAudio.valoresDe(inicio: compassoEmAmostra.comeco, fim: compassoEmAmostra.fim)
                    
                    if (!buffer.isEmpty) {
                        
                        var frequencias = [[Pico]]()
                        
                        //Reconhece frequências de cada janela.
                        //Divide um compasso em janelas de janelamentoSeg.
                        let janelas = Vetor().dividir(entrada: buffer, compPedaco: janelamento, passo: nil)
                        
                        //Processa e encontra as frequências de cada janela.
                        janelas.forEach { frequencias.append(analisador.frequenciasEm(entrada: $0)) }
                        
                        var notas = [(Int, Int, Int)]()
                        
                        //Reconhece notas no compasso
                        let frequenciasPorTempo = dividirTempo(tempo: configAtual.tempoCompasso, frequencias: frequencias, duracaoCompasso: compassoEmAmostra.tamanho)
                        
                        let possiveisFrequencias = self.possiveisFrequencias(frequencias: frequenciasPorTempo)
                        
                        possiveisFrequencias.forEach { filtro in
                            let variacoes = janelasParaFloatArray(filtro: filtro, janelas: frequencias)
                            
                            let notasCruas = analisadorAtaque.ataquesEm(entrada: variacoes)
                            
                            notasCruas.forEach {
                                let comeco = conversor.amostrasParaTicks(amostras: $0.0 * janelamento)
                                let fim = conversor.amostrasParaTicks(amostras: $0.1 * janelamento)
                                notas.append((filtro + 24,
                                              compassoEmTicks.comeco + comeco,
                                              compassoEmTicks.comeco + fim))
                            }
                        }
                        
                        monofonizar(notas: &notas)
                        
                        adicionarDados.serve(status: Engine.Status.Dados, notas)
                        
                        print(tag, "Compasso Terminado")
                    } else {
                        //Indica que o composse foi parado.
                        if (compassoEmAmostra.fim >= valorFinal) {
                            break finalizar
                        }
                        
                        print(tag, "Compasso indisponível")
                        usleep(200000)
                    }
                } while (buffer.isEmpty)
                
                compassoId += 1
            }
        }
        
        adicionarDados.serve(status: Engine.Status.Terminou, "Terminado!")
        print(tag, "Terminou compor!")
    }
    
    func comecar(configuracoes: [ConfiguracaoCompasso]) {
        if (estaProcessando) {
            print(tag, "Já está em processamento!")
            return
        }
        
        DispatchQueue.global(qos: .background).async {
            self.compor(configuracoes: configuracoes)
        }
    }
    
    func interromper(valor: Int) {
        valorFinal = valor
    }
}
