//
//  ConversorTicks.swift
//  ProjetoDeTeste
//
//  Created by Raimundo Corrêa de Oliveira on 09/10/2018.
//  Copyright © 2018 Raimundo Corrêa de Oliveira. All rights reserved.
//
import Foundation

/**
 * Conversor de ticks tem como proposito converter ticks para amostras e amostras pra ticks.
 *
 * @property notaBase define a taxa de amostragem usado nos cálculos.
 * @property metronomo define a potência de 2 que sera transformada.
 * @property freqRange define as configuracao necessarias para conversao de frequencia em nota.
 * @property valorEnergiaDeDescarte define a energia mínima que um pico deve ter.
 * @property dbLimitante define a diferença mínima entre a maior energia e um energia qualquer.
 *
 * @constructor cria um analisador de frequencia para as configuracoes dadas.
 */
class ConversorTicks {
    private var notaBase: Float
    private var metronomo: Float
    private let taxaAmostragem: Float
    //Taxa para conversão
    private var ticksPorAmostra: Float
    
    init(notaBase: Int, metronomo: Int, taxaAmostragem: Int) {
        self.notaBase = Float(notaBase)
        self.metronomo = Float(metronomo)
        self.taxaAmostragem = Float(taxaAmostragem)
        
        self.ticksPorAmostra = 0
        self.ticksPorAmostra = self.calcularTicksPorAmostra()
    }
    
    private func calcularTicksPorAmostra() -> Float {
        let segundosEmMinuto: Float = 60
        let numeroDeTicksPorSegundo = metronomo * notaBase / segundosEmMinuto
        return numeroDeTicksPorSegundo / taxaAmostragem
    }
    
    func amostrasParaTicks(amostras: Int) -> Int {
        return Int(round(ticksPorAmostra * Float(amostras)))
    }
    
    func ticksParaAmostras(ticks: Int) -> Int {
        return Int(round(Float(ticks) / ticksPorAmostra))
    }
    
    func alterar(notaBase: Int, metronomo: Int) {
        self.notaBase = Float(notaBase)
        self.metronomo = Float(metronomo)
        ticksPorAmostra = calcularTicksPorAmostra()
    }
}
