import UIKit
import Accelerate

class DSP {
    private let comprimentoFourier: Int
    
    init(comprimentoFourier: Int) {
        self.comprimentoFourier = comprimentoFourier
    }

    func transformar(signal : [Float]) -> [Float] {
        //Real and imaginary window vector instance
        var signalReal = signal
        signalReal.append(contentsOf: [Float](repeating: 0, count: comprimentoFourier - signal.count))
        
        var signalImaginary = [Float](repeating: 0, count: comprimentoFourier)
        var magnitude = [Float](repeating: 0, count: comprimentoFourier)
        
        //Settings for FFT
        let length = vDSP_Length(floor(log2(Float(comprimentoFourier))))
        let radix = FFTRadix(kFFTRadix2)
        let weights = vDSP_create_fftsetup(length, radix)
        
        signalReal.withUnsafeMutableBufferPointer { forwardInputRealPtr in
            signalImaginary.withUnsafeMutableBufferPointer { forwardInputImagPtr in
              //Real and imaginary window representative
              var splitComplexSignal = DSPSplitComplex(realp: forwardInputRealPtr.baseAddress!, imagp: forwardInputImagPtr.baseAddress!)
                
                vDSP_fft_zip(weights!, &splitComplexSignal, 1, length, FFTDirection(FFT_FORWARD))
                
                vDSP_zvmags(&splitComplexSignal, 1, &magnitude, 1, vDSP_Length(comprimentoFourier))
                
                magnitude = magnitude.map(){sqrt($0)}
                
                vDSP_destroy_fftsetup(weights)
          }
        }
        
        return magnitude
    }
}
