//
//  Filtragem.swift
//  Music-Mate-New-Design
//
//  Created by LTMD on 23/08/2018.
//  Copyright © 2018 LT Music Developer. All rights reserved.
//

import Foundation

/**
 * Classe para filtragem de sinais.
 *
 * @property taxaAmostragem a taxa de amostragem usada pada o processamento.
 * @property comprimento o comprimento do filtro a ser utilizado.
 * @property filtro o gerador de filtros para processamento.
 */
class Filtragem {
    private let taxaAmostragem: Int
    private let comprimento: Int
    
    init (taxaAmostragem: Int, comprimento: Int) {
        self.taxaAmostragem = taxaAmostragem
        self.comprimento = comprimento
    }
    func downSampling(entrada: [Float], deTaxa: Int, paraTaxa: Int) -> [Float] {
        let taxaAmostragemInicial = deTaxa
        let taxaAmostragemFinal = paraTaxa
        
        let fator = taxaAmostragemInicial / taxaAmostragemFinal
        
        if (fator <= 0 || taxaAmostragemFinal * fator != taxaAmostragemInicial) {
            return entrada
        }
        
        var saida = [Float](repeating: 0, count: entrada.count / fator)
        
        var passoSimples = 0
        var passoPorFator = 0
        while passoSimples < saida.count && passoPorFator < entrada.count {
            saida[passoSimples] = entrada[passoPorFator]
            passoSimples += 1
            passoPorFator += fator
        }
        
        return saida
    }
}
